
package com.mycompany.msgsenderandreceiver.SenderApi;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.annotation.Resource;
import javax.ejb.SessionContext;
import javax.ejb.Stateless;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.DeliveryMode;
import javax.jms.MessageProducer;
import javax.jms.ObjectMessage;
import javax.jms.Queue;
import javax.jms.Session;
import org.apache.log4j.Logger;

@Stateless
public class SenderImpl implements SenderApi {
    protected static final Logger logger = Logger.getLogger(SenderImpl.class);
    
    @Resource(mappedName = "java:/ConnectionFactory")
    private ConnectionFactory factory;

    @Resource(mappedName = "java:/queue/TestQueue")
    private Queue testQueue;
    
    @Resource
    SessionContext sessionContext;

    @Override
    public void sendMessage(Serializable data) {
        sendMessageToQueue(data);       
    }
    
    protected void sendMessageToQueue(Serializable data) {
        logger.info("sending message to queue");
        Connection connection = null;
        Session session = null;
        MessageProducer messageProducer = null;

        try {
            connection = factory.createConnection();
            session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
            messageProducer = session.createProducer(testQueue);
            
            // Optimization to message, because we don't use these
            messageProducer.setDisableMessageID(true);
            messageProducer.setDisableMessageTimestamp(true);

            ObjectMessage objectMessage = session.createObjectMessage();
           
            objectMessage.setObject(data);
                //set operation type. Need to pass this as a String due to lousy setObjectMessage definition of JMS spec.
            /*objectMessage.setObjectProperty(Constants.OPERATION, operation.toString());
            objectMessage.setStringProperty(Constants.SESSIONID, tid.getSessionID());
            objectMessage.setLongProperty(Constants.SESSIONTIME, tid.getTime());
            objectMessage.setIntProperty(Constants.FEEDBACKCOUNT,tid.getFeedbackCounter());
            //set approval related paremeter
            objectMessage.setIntProperty(Constants.APPROVAL_TASK_TARGET_ID, tid.getApprovalTaskTargetId());
            objectMessage.setIntProperty(Constants.APPROVAL_TASK_ID, tid.getApprovalTaskId());
            objectMessage.setStringProperty(Constants.EXECUTING_USER, tid.getUser());
            
            // set mass operation related parameters
            objectMessage.setBooleanProperty(Constants.ISMASSOPERATION, true);*/

            messageProducer.send(objectMessage, DeliveryMode.NON_PERSISTENT, 9, messageProducer.getTimeToLive());
        }
        catch (Exception jmse) {
            logger.error("JMS message sending failed ", jmse);
        }
        finally {
            if (session != null) {
                try {
                    session.close();
                } catch (Exception e) {
                    logger.warn("Cannot close session", e);
                }
            }
            if (connection != null) {
                try {
                    connection.close();
                } catch (Exception jmsex) {
                    logger.warn("Failed to close JMS Connection", jmsex);
                }
            }
        }
    }

    
    
}
