package com.mycompany.msgsenderandreceiver.data;

import java.util.StringTokenizer;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlTransient;
import javax.xml.bind.annotation.adapters.XmlAdapter;


@XmlAccessorType(XmlAccessType.FIELD)
public abstract class AbstractOrganisationDTO extends AuditableDto implements Comparable {

    @XmlTransient
    protected Integer dbId;
    
    @XmlTransient
    protected int organisationLevel1Id;
    
    @XmlTransient
    protected int organisationLevel2Id;
    
    @XmlTransient
    protected int organisationLevel3Id;
    
    @XmlTransient
    protected int organisationLevel4Id;
    
    @XmlTransient
    protected int organisationLevel5Id;
    
    @XmlTransient
    protected int organisationLevel6Id;
    
    protected String organisationIdentifier;
    protected String name;
    
    protected String comment;
    
    protected AbstractOrganisationDTO() {
    }

    protected AbstractOrganisationDTO(Integer[] orgBlockIdArray) {
        this.organisationLevel1Id = orgBlockIdArray[0];
        this.organisationLevel2Id = orgBlockIdArray[1];
        this.organisationLevel3Id = orgBlockIdArray[2];
        this.organisationLevel4Id = orgBlockIdArray[3];
        this.organisationLevel5Id = orgBlockIdArray[4];
        this.organisationLevel6Id = orgBlockIdArray[5];
        this.organisationIdentifier = buildOrganisationIdentifier();
    }

    protected AbstractOrganisationDTO(int organisationLevel1Id, int organisationLevel2Id, int organisationLevel3Id, int organisationLevel4Id, int organisationLevel5Id, int organisationLevel6Id) {
        this.organisationLevel1Id = organisationLevel1Id;
        this.organisationLevel2Id = organisationLevel2Id;
        this.organisationLevel3Id = organisationLevel3Id;
        this.organisationLevel4Id = organisationLevel4Id;
        this.organisationLevel5Id = organisationLevel5Id;
        this.organisationLevel6Id = organisationLevel6Id;
        this.organisationIdentifier = buildOrganisationIdentifier();
    }
    
    protected AbstractOrganisationDTO(String organisationIdentifier) {
        this.organisationIdentifier = organisationIdentifier;
        parseAndSetLevelIdsFromOrganisationIdentifier();
    }
    
    protected AbstractOrganisationDTO(String organisationIdentifier, boolean validate) throws Exception {
        if(validate){
            validateIdentifier(organisationIdentifier);
        }
        this.organisationIdentifier = organisationIdentifier;
        parseAndSetLevelIdsFromOrganisationIdentifier();
    }
    
    @Override
    public AbstractOrganisationDTO getOrganisation() {
        return this;
    }
    
    
    public static void validateIdentifier(String identifier) throws Exception{
        
        if (identifier != null) {
            StringTokenizer tok = new StringTokenizer(identifier, "-");
            int[] ids = new int[6];

            for (int i = 0; i < ids.length; i++) {
                ids[i] = -1;
            }

            int i = 0;
            if (!identifier.equals("-1")) {

                while (tok.hasMoreTokens() && i < 6) {
                    String s = tok.nextToken();

                    ids[i] = Integer.parseInt(s);
                    i++;
                }
            }
                if(tok.hasMoreTokens() && i==6){
                throw new Exception("Only6levels");
            }

        }
    }
    
    public final void parseAndSetLevelIdsFromOrganisationIdentifier() {
        if (organisationIdentifier != null) {
            StringTokenizer tok = new StringTokenizer(organisationIdentifier, "-");
            int[] ids = new int[6];

            for (int i = 0; i < ids.length; i++) {
                ids[i] = -1;
            }

            if (!organisationIdentifier.equals("-1")) {
                int i = 0;

                while (tok.hasMoreTokens() && i < 6) {
                    String s = tok.nextToken();

                    ids[i] = Integer.parseInt(s);
                    i++;
                }
            }

            this.organisationLevel1Id = ids[0];
            this.organisationLevel2Id = ids[1];
            this.organisationLevel3Id = ids[2];
            this.organisationLevel4Id = ids[3];
            this.organisationLevel5Id = ids[4];       
            this.organisationLevel6Id = ids[5];
        }
    }        

    

    public boolean isRefOrganisation() {
        return     organisationLevel1Id == -1
                && organisationLevel2Id == -1
                && organisationLevel3Id == -1
                && organisationLevel4Id == -1
                && organisationLevel5Id == -1
                && organisationLevel6Id == -1;
    }

    public boolean isLevel1Organisation() {
        return     organisationLevel1Id != -1
                && organisationLevel2Id == -1
                && organisationLevel3Id == -1
                && organisationLevel4Id == -1
                && organisationLevel5Id == -1
                && organisationLevel6Id == -1;
    }

    @Override
    public Integer getDbId() {
        return dbId;
    }

    @Override
    public void setDbId(Integer dbId) {
        this.dbId = dbId;
    }

    public boolean idsMatch(AbstractOrganisationDTO org) {
        if(organisationLevel1Id == org.organisationLevel1Id
        && organisationLevel2Id == org.organisationLevel2Id
        && organisationLevel3Id == org.organisationLevel3Id
        && organisationLevel4Id == org.organisationLevel4Id
        && organisationLevel5Id == org.organisationLevel5Id
        && organisationLevel6Id == org.organisationLevel6Id){
            return true;
        }
        return false;
    }

    public Integer[] getOrganisationIdAsArray() {
        Integer[] orgBlockIdArray = new Integer[6];
        orgBlockIdArray[0] = this.organisationLevel1Id;
        orgBlockIdArray[1] = this.organisationLevel2Id;
        orgBlockIdArray[2] = this.organisationLevel3Id;
        orgBlockIdArray[3] = this.organisationLevel4Id;
        orgBlockIdArray[4] = this.organisationLevel5Id;
        orgBlockIdArray[5] = this.organisationLevel6Id;
        return orgBlockIdArray;
    }

    public int getOrganisationLevel1Id() {
        return organisationLevel1Id;
    }

    public final void setOrganisationLevel1Id(int organisationLevel1Id) {
        this.organisationLevel1Id = organisationLevel1Id;
        this.organisationIdentifier = buildOrganisationIdentifier();
    }

    public int getOrganisationLevel2Id() {
        return organisationLevel2Id;
    }

    public final void setOrganisationLevel2Id(int organisationLevel2Id) {
        this.organisationLevel2Id = organisationLevel2Id;
        this.organisationIdentifier = buildOrganisationIdentifier();
    }

    public int getOrganisationLevel3Id() {
        return organisationLevel3Id;
    }

    public final void setOrganisationLevel3Id(int organisationLevel3Id) {
        this.organisationLevel3Id = organisationLevel3Id;
        this.organisationIdentifier = buildOrganisationIdentifier();
    }

    public int getOrganisationLevel4Id() {
        return organisationLevel4Id;
    }

    public final void setOrganisationLevel4Id(int organisationLevel4Id) {
        this.organisationLevel4Id = organisationLevel4Id;
        this.organisationIdentifier = buildOrganisationIdentifier();
    }

    public int getOrganisationLevel5Id() {
        return organisationLevel5Id;
    }

    public final void setOrganisationLevel5Id(int organisationLevel5Id) {
        this.organisationLevel5Id = organisationLevel5Id;
        this.organisationIdentifier = buildOrganisationIdentifier();
    }

    public int getOrganisationLevel6Id() {
        return organisationLevel6Id;
    }

    public final void setOrganisationLevel6Id(int organisationLevel6Id) {
        this.organisationLevel6Id = organisationLevel6Id;
        this.organisationIdentifier = buildOrganisationIdentifier();
    }
    
    public String getOrgIdentifierAsString() {
        return getOrganisationIdentifier();
    }

    public String getOrganisationIdentifier() {
        // return organisationIdentifier;
        return buildOrganisationIdentifier();
    }

    public void setOrganisationIdentifier(String organisationIdentifier) {
        this.organisationIdentifier = organisationIdentifier;
        parseAndSetLevelIdsFromOrganisationIdentifier();
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getComment() {
        return comment;
    }

    public void setComment(String comment) {
        this.comment = comment;
    }

    public int getNodeId() {
        int level = getNodeLevel();

        if (level == 1) {
            return this.getOrganisationLevel1Id();
        } else if (level == 2) {
            return this.getOrganisationLevel2Id();
        } else if (level == 3) {
            return this.getOrganisationLevel3Id();
        } else if (level == 4) {
            return this.getOrganisationLevel4Id();
        } else if (level == 5) {
            return this.getOrganisationLevel5Id();
        } else if (level == 6) {
            return this.getOrganisationLevel6Id();
        } else {
            return -1;
        }
    }

    private String buildOrganisationIdentifier() {
        String identifierValue;

        int level = getNodeLevel( organisationLevel1Id, organisationLevel2Id, organisationLevel3Id, organisationLevel4Id, organisationLevel5Id, organisationLevel6Id);
        identifierValue = organisationLevel1Id + "";

        if (level >= 2) {
            identifierValue += "-" + organisationLevel2Id;
        }

        if (level >= 3) {
            identifierValue += "-" + organisationLevel3Id;
        }

        if (level >= 4) {
            identifierValue += "-" + organisationLevel4Id;
        }

        if (level >= 5) {
            identifierValue += "-" + organisationLevel5Id;
        }

        if (level >= 6) {
            identifierValue += "-" + organisationLevel6Id;
        }

        return identifierValue;
    }
    
    public int getNodeLevel() {
        return getNodeLevel(getOrganisationLevel1Id(), getOrganisationLevel2Id(), getOrganisationLevel3Id(),
                            getOrganisationLevel4Id(), getOrganisationLevel5Id(), getOrganisationLevel6Id());
    }

    public static int getNodeLevel(int organisationLevel1Id, int organisationLevel2Id, int organisationLevel3Id,
                                   int organisationLevel4Id, int organisationLevel5Id, int organisationLevel6Id) {
        int i = 0;
        if (organisationLevel1Id != -1) {
            i++;
        }
        if (organisationLevel2Id != -1) {
            i++;
        }
        if (organisationLevel3Id != -1) {
            i++;
        }
        if (organisationLevel4Id != -1) {
            i++;
        }
        if (organisationLevel5Id != -1) {
            i++;
        }
        if (organisationLevel6Id != -1) {
            i++;
        }
        return i;
    }
    
    public int[] getIdentifiersAsArray()
    {
        int[] array= {organisationLevel1Id,organisationLevel2Id,organisationLevel3Id,organisationLevel4Id,organisationLevel5Id,organisationLevel6Id};
        return array;
    }

    @Override
    public int compareTo(Object obj) {
        if (obj == null) {
            return -1;
        }
        else if (getClass() != obj.getClass()) {
            return -1;
        }

        int result = 0;
        final AbstractOrganisationDTO other = (AbstractOrganisationDTO) obj;
        
        result = Integer.valueOf(this.organisationLevel1Id).compareTo(other.organisationLevel1Id);

        if (result == 0) {
            result = Integer.valueOf(this.organisationLevel2Id).compareTo(other.organisationLevel2Id);
        }

        if (result == 0) {
            result = Integer.valueOf(this.organisationLevel3Id).compareTo(other.organisationLevel3Id);
        }

        if (result == 0) {
            result = Integer.valueOf(this.organisationLevel4Id).compareTo(other.organisationLevel4Id);
        }

        if (result == 0) {
            result = Integer.valueOf(this.organisationLevel5Id).compareTo(other.organisationLevel5Id);
        }

        if (result == 0) {
            result = Integer.valueOf(this.organisationLevel6Id).compareTo(other.organisationLevel6Id);
        }

        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (!(obj instanceof AbstractOrganisationDTO)) {
            return false;
        }
        final AbstractOrganisationDTO other = (AbstractOrganisationDTO) obj;
        if (this.organisationLevel1Id != other.organisationLevel1Id) {
            return false;
        }
        if (this.organisationLevel2Id != other.organisationLevel2Id) {
            return false;
        }
        if (this.organisationLevel3Id != other.organisationLevel3Id) {
            return false;
        }
        if (this.organisationLevel4Id != other.organisationLevel4Id) {
            return false;
        }
        if (this.organisationLevel5Id != other.organisationLevel5Id) {
            return false;
        }
        if (this.organisationLevel6Id != other.organisationLevel6Id) {
            return false;
        }
        return true;
    }
    
    @Override
    public int hashCode() {
        int hash = 7;
        hash = 13 * hash + this.organisationLevel1Id;
        hash = 13 * hash + this.organisationLevel2Id;
        hash = 13 * hash + this.organisationLevel3Id;
        hash = 13 * hash + this.organisationLevel4Id;
        hash = 13 * hash + this.organisationLevel5Id;
        hash = 13 * hash + this.organisationLevel6Id;
        return hash;
    }

    public boolean isParentOf(AbstractOrganisationDTO anotherOrg)
    {
        boolean iAmParent=false;
        
        int myLevel= getNodeLevel();
        int anotherLevel=anotherOrg.getNodeLevel();
        
        if(anotherLevel>myLevel)
        {
            int myIds[]=getIdentifiersAsArray();
            int anotherIds[]=anotherOrg.getIdentifiersAsArray();
            
            
            for(int i=0; i<myLevel;i++ )
            {
                if(myIds[i]!=anotherIds[i])
                {
                    return false;
                }
            }
            iAmParent=true;
        }
     
        
        return iAmParent;
    }

    public int getLevelId() {
        if (organisationLevel6Id != -1) {
            return organisationLevel6Id;
        }
        else if (organisationLevel5Id != -1) {
            return organisationLevel5Id;
        }
        else if (organisationLevel4Id != -1) {
            return organisationLevel4Id;
        }
        else if (organisationLevel3Id != -1) {
            return organisationLevel3Id;
        }
        else if (organisationLevel2Id != -1) {
            return organisationLevel2Id;
        }
        else if (organisationLevel1Id != -1) {
            return organisationLevel1Id;
        }
        else {
            return -1;
        }
    }

    

    public static class OrganisationReferencingAdapter extends XmlAdapter<String, AbstractOrganisationDTO> {

        @Override
        public String marshal(AbstractOrganisationDTO org) throws Exception {
            if(org!=null){
                if(org.getOrgIdentifierAsString().equals("-1")){
                    return "*";
                }else{
                    return org.getOrgIdentifierAsString();
                }
            }else{
                return null;
            }
//            return (org == null) ? null : org.getOrganisationIdentifier();
        }

        @Override
        public AbstractOrganisationDTO unmarshal(String organisationIdentifier) throws Exception {
            
            // Here we return always Tetra organisation, because it fits to all object types,
            // type doesn't have to be real one, because full organisation is fetched anyway in backupbean.
            if(organisationIdentifier.equalsIgnoreCase("*")){
                return new OrganisationTetraDTO("-1");
            }else{
                return new OrganisationTetraDTO(organisationIdentifier);
            }
        }
    }  

    public static class ExcludeOrganisationReferencingAdapter extends XmlAdapter<String, AbstractOrganisationDTO> {
        private boolean exclude;

        public ExcludeOrganisationReferencingAdapter() {
            this.exclude = false;
        }
        
        public ExcludeOrganisationReferencingAdapter(boolean exclude) {
            this.exclude = exclude;
        }
        
        @Override
        public String marshal(AbstractOrganisationDTO org) throws Exception {
            if (exclude == true) {
                return null;
            }
            else if (org != null) {
                if (org.getOrgIdentifierAsString().equals("-1")) {
                    return "*";
                }
                else {
                    return org.getOrgIdentifierAsString();
                }
            } else {
                return null;
            }
        }

        @Override
        public AbstractOrganisationDTO unmarshal(String organisationIdentifier) throws Exception {
            if (organisationIdentifier.equalsIgnoreCase("*")) {
                return new OrganisationTetraDTO("-1");
            } else {
                return new OrganisationTetraDTO(organisationIdentifier);
            }
        }
    }  
    


    
}

