package com.mycompany.msgsenderandreceiver.data;

import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import javax.xml.bind.annotation.adapters.XmlAdapter;

/**
 *
 * @author Ari Vanajoki, Tieto Oyj
 */
public class DxtDTO extends ComparableDTO implements  Comparable<DxtDTO>
{
    private static final long serialVersionUID = 1L;
    
    private Integer dxtId;
    private String dxtName;
    private short status;

    public Integer getDxtId() {
        return dxtId;
    }

    public String getDxtIdAsHex() {
        if (dxtId == null) {
            return "null";
        }
        else {
            String asHex = Integer.toHexString(dxtId).toUpperCase();

            while (asHex.length() < 8) {
                asHex = "0" + asHex;
            }

            return asHex + "h";
        }
    }

    public void setDxtId(Integer dxtId) {
        this.dxtId = dxtId;
    }

    public void setDxtIdAsHex(String dxtIdAsHex) {
        if (dxtIdAsHex == null) {
            dxtId = null;
        }
        else {
            String temp = "0x" + dxtIdAsHex.substring(0, dxtIdAsHex.length() - 1);
            dxtId = Integer.decode(temp);
        }
    }

    public String getDxtName() {
        return dxtName;
    }

    public void setDxtName(String dxtName) {
        this.dxtName = dxtName;
    }

    public short getStatus() {
        return status;
    }

    public void setStatus(short status) {
        this.status = status;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final DxtDTO other = (DxtDTO) obj;
        if (!Objects.equals(this.dxtId, other.dxtId)) {
            return false;
        }
        return true;
    }

    

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 59 * hash + (this.dxtId != null ? this.dxtId.hashCode() : 0);
        return hash;
    }
    
 

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append(dxtName);
        sb.append(" (");
        sb.append(getDxtIdAsHex());
        sb.append(")");
        return sb.toString();
    }
    
    @Override
    public List<String> getCddExcludedFields() {
        List<String> fields = super.getCddExcludedFields();
        fields.addAll(Arrays.asList("dbId", "status"));
        return fields;
    }

    @Override
    public int compareTo(DxtDTO t) {
        
        return this.dxtId - t.getDxtId();
    }

    public static class DxtReferencingAdapter extends XmlAdapter<String, DxtDTO> {

        @Override
        public String marshal(DxtDTO dxt) throws Exception {
            return (dxt == null) ? null : dxt.getDxtIdAsHex();
        }

        @Override
        public DxtDTO unmarshal(String dxtIdAsHex) throws Exception {
            DxtDTO dto = new DxtDTO();
            dto.setDxtIdAsHex(dxtIdAsHex);
            return dto;
        }
    }  
}
